"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var Support_1 = require("./Support");
var stringHash = require("string-hash");
var TypeAttributeKind = /** @class */ (function () {
    function TypeAttributeKind(name, combine) {
        this.name = name;
        if (combine === undefined) {
            combine = function () {
                return Support_1.panic("Cannot combine type attribute " + name);
            };
        }
        this.combine = combine;
    }
    TypeAttributeKind.prototype.makeAttributes = function (value) {
        var kvps = [[this, value]];
        return immutable_1.Map(kvps);
    };
    TypeAttributeKind.prototype.tryGetInAttributes = function (a) {
        return a.get(this);
    };
    TypeAttributeKind.prototype.setInAttributes = function (a, value) {
        return a.set(this, value);
    };
    TypeAttributeKind.prototype.modifyInAttributes = function (a, modify) {
        var modified = modify(this.tryGetInAttributes(a));
        if (modified === undefined) {
            return a.remove(this);
        }
        return this.setInAttributes(a, modified);
    };
    TypeAttributeKind.prototype.setDefaultInAttributes = function (a, makeDefault) {
        if (this.tryGetInAttributes(a) !== undefined)
            return a;
        return this.modifyInAttributes(a, makeDefault);
    };
    TypeAttributeKind.prototype.equals = function (other) {
        if (!(other instanceof TypeAttributeKind)) {
            return false;
        }
        return this.name === other.name;
    };
    TypeAttributeKind.prototype.hashCode = function () {
        return stringHash(this.name);
    };
    return TypeAttributeKind;
}());
exports.TypeAttributeKind = TypeAttributeKind;
function combineTypeAttributes(attributeArray) {
    if (attributeArray.length === 0)
        return immutable_1.Map();
    var first = attributeArray[0];
    var rest = attributeArray.slice(1);
    return first.mergeWith.apply(first, [function (aa, ab, kind) { return kind.combine(aa, ab); }].concat(rest));
}
exports.combineTypeAttributes = combineTypeAttributes;
function combineDescriptions(a, b) {
    return a.trim() + "\n\n" + b.trim();
}
exports.descriptionTypeAttributeKind = new TypeAttributeKind("description", combineDescriptions);
exports.propertyDescriptionsTypeAttributeKind = new TypeAttributeKind("propertyDescriptions", function (a, b) { return a.mergeWith(combineDescriptions, b); });
